// Demoprogramm fr die DLL "DfCom.Dll"
// Die DLL ermglicht den Zugriff auf Datafox AE III Terminal 
// ber die serielle Schnittstelle
// 
// Version 1.0 20.11.1999 R. Keller
//
// Version 1.1 7.6.2000 R. Keller
//		MDE Start/Stop erzeugt Datenstze mit Kennung 11 bzw. 12

#include <stdio.h>
#include <conio.h>

// Funktionen die von der DLL "DfCom.Dll" untersttzt werden

extern "C" __declspec(dllimport) int __stdcall Init(int ComNr);
// Schnittstelle zum Datafox AE initialisieren
// ComNr -> Nr. der Schnittstelle; 1 -> COM1; 2 -> COM2 ...
// Return	0 -> Fehler (COM nicht vorhanden; COM bereits belegt ...)
//			1 -> OK

extern "C" __declspec(dllexport) int __stdcall 
	GetDatenOnline(int BusNum, int *Kennung, char *DateTime,int *Z1,int *Z2,int *Z3,int *Z4,int *Z5,int *Z6);
// Datafox AE mit der Busnummer "BusNum" pollen.
// Wenn ein Datensatz im Datafox vorhanden ist, ist der Rckgabewert "1" und
// in den Var. Kennungen, DateTime, Z1 ... Z6 wird der Datensatz zurckgegeben.
// Wurde ein Datensatz mit "GetDatenOnline" abgefragt, wird er nicht im internen Ringspeicher
// des Datafox abgelegt und kann nicht mit "GetDatenOffline" erneut abgeholt werden!
// BusNum	-> Busnummer des Datafox welches abgefragt wird
// Die folgenden Variablen haben nur einen gltigen Wert, wenn der Rckgabewert = "1" ist:
//		Kennung		-> am Datafox eingegebene Kennung 
//		DateTime	-> Datum und Uhrzeit des Datensatzes. Format: "20.11.1999 17:16:35"
//					   Achtung: der Pointer DateTime mu auf einen ausreichend groen Specherbereich zeigen!
//		Z1...Z6		-> Werte der Z-Felder (0 wenn Z-Feld im Datafox nicht verwendet wird)
// Return 1	-> Datensatz vorhanden
// Return 0 -> kein Datensatz vorhanden
// Return 2 -> Fehler: keine Kommunikation

extern "C" __declspec(dllexport) void __stdcall 
	SendMessageOnline(int BusNum, int ErrFlag, char *Message);
// Text an Datafox senden.
// Die Funtion wird aufgerufen nachdem mit "GetDatenOnline" ein Datensatz ebgeholt wurde.
// Der bergebene Text wird auf dem Display des Datafox eine gewisse Zeit angezeitgt.
// Ist das Flag "ErrFlag" gesetzt, wird zustzlich mit dem Summer des Datafox eine Fehler signalisiert.
// Der Text wird nur angezeigt, wenn die Funktion innerhalb eines Zeitfensters nach "GetDatenOnline"
// aufgerufen wird.
// BusNum	-> Busnummer des Datafox
// ErrFlag	->	1 -> Summer aktivieren
//				0 -> Summer nicht aktivieren
// Return	-> kein Rckgabewert

extern "C" __declspec(dllexport) int __stdcall 
	GetDatenOffline(int BusNum, int *Kennung, char *DateTime, int *Z1, int *Z2, int *Z3, int *Z4, int *Z5, int *Z6);
// Im Datafox gespeicherte Daten auslesen
// Wenn im Onlinebetrieb der Datensatz nach einer definierten Zeit nicht abgeholt wurde (mit GetDatenOnline),
// wird der Datensatz im internen Ringspeicher abgelegt (Notlauffunktion).
// Ein Annehmen oder Ablehnen des Datensatzes ist dann nicht mehr mglich.
// BusNum	-> Busnummer des Datafox welches abgefragt wird
// Die folgenden Variablen haben nur einen gltigen Wert, wenn der Rckgabewert = "1" ist:
//		Kennung		-> am Datafox eingegebene Kennung 
//		DateTime	-> Datum und Uhrzeit des Datensatzes Format: "20.11.1999 17:16:35"
//					   Achtung: der Pointer DateTime mu auf einen ausreichend groen Specherbereich zeigen!
//		Z1...Z6		-> Werte der Z-Felder (0 wenn Z-Feld im Datafox nicht verwendet wird)
// Return 1	-> Datensatz erfolgreich abgefragt
// Return 0 -> kein Datensatz vorhanden
// Return 2 -> Fehler: keine Kommunikation

int main(void)
{
	int Kennung,Z1,Z2,Z3,Z4,Z5,Z6;
	char DT[30];
	int i,ret;

	if (Init(2)) {		// Schnittstelle COM1 ffnen
		printf ("Schnittstelle initialisiert\n");
	}else{
		printf ("Fehler Initialisierung der Schnittstelle\n"); getchar(); return 1;
	}

	// Daten Offline lesen
	while (GetDatenOffline(1,&Kennung,DT,&Z1,&Z2,&Z3,&Z4,&Z5,&Z6)==1) {
		printf("Datensatz Offline Kennung:%d  %s\n",Kennung,DT);
		printf("  Z1:%7d   Z2:%7d   Z3:%7d   Z4:%7d   Z5:%7d   Z6:%7d\n\n",Z1,Z2,Z3,Z4,Z5,Z6);  
	}
	// Daten Online lesen bis eine Taste bettigt wird
	printf("Daten lesen Online\n");
	while (!_kbhit()) {
		for (i=1; i<=1; i++) {  // Pollen von Gert Nr. 1 bis 1
			if ((ret=GetDatenOnline(i,&Kennung,DT,&Z1,&Z2,&Z3,&Z4,&Z5,&Z6))==1) {
				// Zum Test werden Datenstze mit Kennung 3 und Z1 < 100 oder Z1 > 999 nicht angenommen
				if (Kennung == 1 && (Z1 < 100 || Z1 > 999)) {
					SendMessageOnline(i,1,"Datensatz\rabgelehnt\rFalsch: Z-Feld 1\r-> Enter"); // Datensatz abgelehnt
					printf("Datensatz Online abgelehnt. Bus-Nr. %d Kennung:%d  %s  Z1:%d\n",i,Kennung,DT,Z1);
				}else if (Kennung >= 1 && Kennung <= 10) { // Datensatz angenommen
					SendMessageOnline(i,0,"\rDaten gespeichert\r-> Enter");
					printf("Datensatz Online Bus-Nr. %d Kennung:%d  %s\n",i,Kennung,DT);
					printf("  Z1:%7d   Z2:%7d   Z3:%7d   Z4:%7d   Z5:%7d   Z6:%7d\n\n",Z1,Z2,Z3,Z4,Z5,Z6);  
				}else if (Kennung == 11 || Kennung == 12) {	// MDE Kennungen
					// neu bei Version 5.07 der Datafox AEIII Firmware
					// MDE Start Stop hat die Kennungen 11 und 12
					// Bitte beachten:
					//    Nach dem Abholen eines Online-Datensatzes wird KEINE Message geschickt!
					printf("MDE Datensatz Online Bus-Nr. %d Kennung:%d  %s\n",i,Kennung,DT);
					printf("  Z1:%7d   Z2:%7d   Z3:%7d   Z4:%7d   Z5:%7d   Z6:%7d\n\n",Z1,Z2,Z3,Z4,Z5,Z6);  
				}
			}else if (ret == 2) {
				printf("Datafox antwortet nicht\n");
			}
		}
	}
	return 0;
}